@@ -4,6 +4,7 @@
 
 #include <obs.hpp>
 #include <util/platform.h>
+#include <util/util_uint64.h>
 #include <graphics/vec4.h>
 #include <graphics/graphics.h>
 #include <graphics/math-extra.h>
@@ -15,7 +16,7 @@
 
 #include "ui_AutoConfigTestPage.h"
 
-#define wiz reinterpret_cast<AutoConfig*>(wizard())
+#define wiz reinterpret_cast<AutoConfig *>(wizard())
 
 using namespace std;
 
@@ -31,17 +32,14 @@ class TestMode {
 		gs_eparam_t *randomvals[3] = {
 			gs_effect_get_param_by_name(solid, "randomvals1"),
 			gs_effect_get_param_by_name(solid, "randomvals2"),
-			gs_effect_get_param_by_name(solid, "randomvals3")
-		};
+			gs_effect_get_param_by_name(solid, "randomvals3")};
 
 		struct vec4 r;
 
 		for (int i = 0; i < 3; i++) {
-			vec4_set(&r,
-				rand_float(true) * 100.0f,
-				rand_float(true) * 100.0f,
-				rand_float(true) * 50000.0f + 10000.0f,
-				0.0f);
+			vec4_set(&r, rand_float(true) * 100.0f,
+				 rand_float(true) * 100.0f,
+				 rand_float(true) * 50000.0f + 10000.0f, 0.0f);
 			gs_effect_set_vec4(randomvals[i], &r);
 		}
 
@@ -86,37 +84,37 @@ public:
 
 /* ------------------------------------------------------------------------- */
 
-#define TEST_STR(x)             "Basic.AutoConfig.TestPage." x
-#define SUBTITLE_TESTING        TEST_STR("Subtitle.Testing")
-#define SUBTITLE_COMPLETE       TEST_STR("Subtitle.Complete")
-#define TEST_BW                 TEST_STR("TestingBandwidth")
-#define TEST_BW_CONNECTING      TEST_STR("TestingBandwidth.Connecting")
-#define TEST_BW_CONNECT_FAIL    TEST_STR("TestingBandwidth.ConnectFailed")
-#define TEST_BW_SERVER          TEST_STR("TestingBandwidth.Server")
-#define TEST_RES                TEST_STR("TestingRes")
-#define TEST_RES_VAL            TEST_STR("TestingRes.Resolution")
-#define TEST_RES_FAIL           TEST_STR("TestingRes.Fail")
-#define TEST_SE                 TEST_STR("TestingStreamEncoder")
-#define TEST_RE                 TEST_STR("TestingRecordingEncoder")
-#define TEST_RESULT_SE          TEST_STR("Result.StreamingEncoder")
-#define TEST_RESULT_RE          TEST_STR("Result.RecordingEncoder")
+#define TEST_STR(x) "Basic.AutoConfig.TestPage." x
+#define SUBTITLE_TESTING TEST_STR("Subtitle.Testing")
+#define SUBTITLE_COMPLETE TEST_STR("Subtitle.Complete")
+#define TEST_BW TEST_STR("TestingBandwidth")
+#define TEST_BW_CONNECTING TEST_STR("TestingBandwidth.Connecting")
+#define TEST_BW_CONNECT_FAIL TEST_STR("TestingBandwidth.ConnectFailed")
+#define TEST_BW_SERVER TEST_STR("TestingBandwidth.Server")
+#define TEST_RES TEST_STR("TestingRes")
+#define TEST_RES_VAL TEST_STR("TestingRes.Resolution")
+#define TEST_RES_FAIL TEST_STR("TestingRes.Fail")
+#define TEST_SE TEST_STR("TestingStreamEncoder")
+#define TEST_RE TEST_STR("TestingRecordingEncoder")
+#define TEST_RESULT_SE TEST_STR("Result.StreamingEncoder")
+#define TEST_RESULT_RE TEST_STR("Result.RecordingEncoder")
 
 void AutoConfigTestPage::StartBandwidthStage()
 {
 	ui->progressLabel->setText(QTStr(TEST_BW));
-	testThread = std::thread([this] () {TestBandwidthThread();});
+	testThread = std::thread([this]() { TestBandwidthThread(); });
 }
 
 void AutoConfigTestPage::StartStreamEncoderStage()
 {
 	ui->progressLabel->setText(QTStr(TEST_SE));
-	testThread = std::thread([this] () {TestStreamEncoderThread();});
+	testThread = std::thread([this]() { TestStreamEncoderThread(); });
 }
 
 void AutoConfigTestPage::StartRecordingEncoderStage()
 {
 	ui->progressLabel->setText(QTStr(TEST_RE));
-	testThread = std::thread([this] () {TestRecordingEncoderThread();});
+	testThread = std::thread([this]() { TestRecordingEncoderThread(); });
 }
 
 void AutoConfigTestPage::GetServers(std::vector<ServerInfo> &servers)
@@ -157,6 +155,8 @@ static inline void string_depad_key(string &key)
 	}
 }
 
+const char *FindAudioEncoderFromCodec(const char *type);
+
 void AutoConfigTestPage::TestBandwidthThread()
 {
 	bool connected = false;
@@ -174,24 +174,20 @@ void AutoConfigTestPage::TestBandwidthThread()
 	 */
 
 	QMetaObject::invokeMethod(this, "UpdateMessage",
-			Q_ARG(QString, QStringLiteral("")));
+				  Q_ARG(QString, QStringLiteral("")));
 
 	/* -----------------------------------*/
 	/* create obs objects                 */
 
-	const char *serverType = wiz->customServer
-		? "rtmp_custom"
-		: "rtmp_common";
+	const char *serverType = wiz->customServer ? "rtmp_custom"
+						   : "rtmp_common";
 
-	OBSEncoder vencoder = obs_video_encoder_create("obs_x264",
-			"test_x264", nullptr, nullptr);
-	OBSEncoder aencoder = obs_audio_encoder_create("ffmpeg_aac",
-			"test_aac", nullptr, 0, nullptr);
-	OBSService service = obs_service_create(serverType,
-			"test_service", nullptr, nullptr);
-	OBSOutput output = obs_output_create("rtmp_output",
-			"test_stream", nullptr, nullptr);
-	obs_output_release(output);
+	OBSEncoder vencoder = obs_video_encoder_create("obs_x264", "test_x264",
+						       nullptr, nullptr);
+	OBSEncoder aencoder = obs_audio_encoder_create("ffmpeg_aac", "test_aac",
+						       nullptr, 0, nullptr);
+	OBSService service = obs_service_create(serverType, "test_service",
+						nullptr, nullptr);
 	obs_encoder_release(vencoder);
 	obs_encoder_release(aencoder);
 	obs_service_release(service);
@@ -219,10 +215,17 @@ void AutoConfigTestPage::TestBandwidthThread()
 	if (wiz->service == AutoConfig::Service::Twitch) {
 		string_depad_key(key);
 		key += "?bandwidthtest";
+	} else if (wiz->serviceName == "Restream.io" ||
+		   wiz->serviceName == "Restream.io - RTMP") {
+		string_depad_key(key);
+		key += "?test=true";
+	} else if (wiz->serviceName == "Restream.io - FTL") {
+		string_depad_key(key);
+		key += "?test";
 	}
 
 	obs_data_set_string(service_settings, "service",
-			wiz->serviceName.c_str());
+			    wiz->serviceName.c_str());
 	obs_data_set_string(service_settings, "key", key.c_str());
 
 	obs_data_set_int(vencoder_settings, "bitrate", wiz->startingBitrate);
@@ -232,9 +235,9 @@ void AutoConfigTestPage::TestBandwidthThread()
 
 	obs_data_set_int(aencoder_settings, "bitrate", 32);
 
-	OBSBasic *main = reinterpret_cast<OBSBasic*>(App()->GetMainWindow());
-	const char *bind_ip = config_get_string(main->Config(), "Output",
-			"BindIP");
+	OBSBasic *main = reinterpret_cast<OBSBasic *>(App()->GetMainWindow());
+	const char *bind_ip =
+		config_get_string(main->Config(), "Output", "BindIP");
 	obs_data_set_string(output_settings, "bind_ip", bind_ip);
 
 	/* -----------------------------------*/
@@ -246,45 +249,77 @@ void AutoConfigTestPage::TestBandwidthThread()
 	else
 		GetServers(servers);
 
-	/* just use the first server if it only has one alternate server */
-	if (servers.size() < 3)
+	/* just use the first server if it only has one alternate server,
+	 * or if using Restream or Nimo TV due to their "auto" servers */
+	if (servers.size() < 3 ||
+	    wiz->serviceName.substr(0, 11) == "Restream.io" ||
+	    wiz->serviceName == "Nimo TV") {
 		servers.resize(1);
 
+	} else if (wiz->service == AutoConfig::Service::Twitch &&
+		   wiz->twitchAuto) {
+		/* if using Twitch and "Auto" is available, test 3 closest
+		 * server */
+		servers.erase(servers.begin() + 1);
+		servers.resize(3);
+	} else if (wiz->service == AutoConfig::Service::YouTube) {
+		/* Only test first set of primary + backup servers */
+		servers.resize(2);
+	}
+
 	/* -----------------------------------*/
-	/* apply settings                     */
+	/* apply service settings             */
 
 	obs_service_update(service, service_settings);
-	obs_service_apply_encoder_settings(service,
-			vencoder_settings, aencoder_settings);
+	obs_service_apply_encoder_settings(service, vencoder_settings,
+					   aencoder_settings);
 
-	obs_encoder_update(vencoder, vencoder_settings);
-	obs_encoder_update(aencoder, aencoder_settings);
+	/* -----------------------------------*/
+	/* create output                      */
+
+	const char *output_type = obs_service_get_output_type(service);
+	if (!output_type)
+		output_type = "rtmp_output";
+
+	OBSOutput output =
+		obs_output_create(output_type, "test_stream", nullptr, nullptr);
+	obs_output_release(output);
 	obs_output_update(output, output_settings);
 
+	const char *audio_codec = obs_output_get_supported_audio_codecs(output);
+
+	if (strcmp(audio_codec, "aac") != 0) {
+		const char *id = FindAudioEncoderFromCodec(audio_codec);
+		aencoder = obs_audio_encoder_create(id, "test_audio", nullptr,
+						    0, nullptr);
+		obs_encoder_release(aencoder);
+	}
+
 	/* -----------------------------------*/
 	/* connect encoders/services/outputs  */
 
+	obs_encoder_update(vencoder, vencoder_settings);
+	obs_encoder_update(aencoder, aencoder_settings);
 	obs_encoder_set_video(vencoder, obs_get_video());
 	obs_encoder_set_audio(aencoder, obs_get_audio());
 
 	obs_output_set_video_encoder(output, vencoder);
 	obs_output_set_audio_encoder(output, aencoder, 0);
+	obs_output_set_reconnect_settings(output, 0, 0);
 
 	obs_output_set_service(output, service);
 
 	/* -----------------------------------*/
 	/* connect signals                    */
 
-	auto on_started = [&] ()
-	{
+	auto on_started = [&]() {
 		unique_lock<mutex> lock(m);
 		connected = true;
 		stopped = false;
 		cv.notify_one();
 	};
 
-	auto on_stopped = [&] ()
-	{
+	auto on_stopped = [&]() {
 		unique_lock<mutex> lock(m);
 		connected = false;
 		stopped = true;
@@ -294,17 +329,15 @@ void AutoConfigTestPage::TestBandwidthThread()
 	using on_started_t = decltype(on_started);
 	using on_stopped_t = decltype(on_stopped);
 
-	auto pre_on_started = [] (void *data, calldata_t *)
-	{
+	auto pre_on_started = [](void *data, calldata_t *) {
 		on_started_t &on_started =
-			*reinterpret_cast<on_started_t*>(data);
+			*reinterpret_cast<on_started_t *>(data);
 		on_started();
 	};
 
-	auto pre_on_stopped = [] (void *data, calldata_t *)
-	{
+	auto pre_on_stopped = [](void *data, calldata_t *) {
 		on_stopped_t &on_stopped =
-			*reinterpret_cast<on_stopped_t*>(data);
+			*reinterpret_cast<on_stopped_t *>(data);
 		on_stopped();
 	};
 
@@ -325,12 +358,13 @@ void AutoConfigTestPage::TestBandwidthThread()
 
 		int per = int((i + 1) * 100 / servers.size());
 		QMetaObject::invokeMethod(this, "Progress", Q_ARG(int, per));
-		QMetaObject::invokeMethod(this, "UpdateMessage",
-				Q_ARG(QString, QTStr(TEST_BW_CONNECTING)
-					.arg(server.name.c_str())));
+		QMetaObject::invokeMethod(
+			this, "UpdateMessage",
+			Q_ARG(QString, QTStr(TEST_BW_CONNECTING)
+					       .arg(server.name.c_str())));
 
 		obs_data_set_string(service_settings, "server",
-				server.address.c_str());
+				    server.address.c_str());
 		obs_service_update(service, service_settings);
 
 		if (!obs_output_start(output))
@@ -352,9 +386,10 @@ void AutoConfigTestPage::TestBandwidthThread()
 		if (!connected)
 			continue;
 
-		QMetaObject::invokeMethod(this, "UpdateMessage",
-				Q_ARG(QString, QTStr(TEST_BW_SERVER)
-					.arg(server.name.c_str())));
+		QMetaObject::invokeMethod(
+			this, "UpdateMessage",
+			Q_ARG(QString,
+			      QTStr(TEST_BW_SERVER).arg(server.name.c_str())));
 
 		/* ignore first 2.5 seconds due to possible buffering skewing
 		 * the result */
@@ -384,12 +419,14 @@ void AutoConfigTestPage::TestBandwidthThread()
 		cv.wait(ul);
 
 		uint64_t total_time = os_gettime_ns() - t_start;
+		if (total_time == 0)
+			total_time = 1;
 
-		int total_bytes = (int)obs_output_get_total_bytes(output) -
-			start_bytes;
-		uint64_t bitrate = (uint64_t)total_bytes * 8
-			* 1000000000 / total_time / 1000;
-
+		int total_bytes =
+			(int)obs_output_get_total_bytes(output) - start_bytes;
+		uint64_t bitrate = util_mul_div64(
+			total_bytes, 8ULL * 1000000000ULL / 1000ULL,
+			total_time);
 		if (obs_output_get_frames_dropped(output) ||
 		    (int)bitrate < (wiz->startingBitrate * 75 / 100)) {
 			server.bitrate = (int)bitrate * 70 / 100;
@@ -403,7 +440,8 @@ void AutoConfigTestPage::TestBandwidthThread()
 
 	if (!success) {
 		QMetaObject::invokeMethod(this, "Failure",
-				Q_ARG(QString, QTStr(TEST_BW_CONNECT_FAIL)));
+					  Q_ARG(QString,
+						QTStr(TEST_BW_CONNECT_FAIL)));
 		return;
 	}
 
@@ -424,8 +462,8 @@ void AutoConfigTestPage::TestBandwidthThread()
 		}
 	}
 
-	wiz->server = bestServer;
-	wiz->serverName = bestServerName;
+	wiz->server = std::move(bestServer);
+	wiz->serverName = std::move(bestServerName);
 	wiz->idealBitrate = bestBitrate;
 
 	QMetaObject::invokeMethod(this, "NextStage");
@@ -447,7 +485,8 @@ static long double EstimateMinBitrate(int cx, int cy, int fps_num, int fps_den)
 	return EstimateBitrateVal(cx, cy, fps_num, fps_den) / val;
 }
 
-static long double EstimateUpperBitrate(int cx, int cy, int fps_num, int fps_den)
+static long double EstimateUpperBitrate(int cx, int cy, int fps_num,
+					int fps_den)
 {
 	long double val = EstimateBitrateVal(1280, 720, 30, 1) / 3000.0l;
 	return EstimateBitrateVal(cx, cy, fps_num, fps_den) / val;
@@ -487,17 +526,17 @@ bool AutoConfigTestPage::TestSoftwareEncoding()
 {
 	TestMode testMode;
 	QMetaObject::invokeMethod(this, "UpdateMessage",
-			Q_ARG(QString, QStringLiteral("")));
+				  Q_ARG(QString, QStringLiteral("")));
 
 	/* -----------------------------------*/
 	/* create obs objects                 */
 
-	OBSEncoder vencoder = obs_video_encoder_create("obs_x264",
-			"test_x264", nullptr, nullptr);
-	OBSEncoder aencoder = obs_audio_encoder_create("ffmpeg_aac",
-			"test_aac", nullptr, 0, nullptr);
-	OBSOutput output = obs_output_create("null_output",
-			"null", nullptr, nullptr);
+	OBSEncoder vencoder = obs_video_encoder_create("obs_x264", "test_x264",
+						       nullptr, nullptr);
+	OBSEncoder aencoder = obs_audio_encoder_create("ffmpeg_aac", "test_aac",
+						       nullptr, 0, nullptr);
+	OBSOutput output =
+		obs_output_create("null_output", "null", nullptr, nullptr);
 	obs_output_release(output);
 	obs_encoder_release(vencoder);
 	obs_encoder_release(aencoder);
@@ -514,7 +553,7 @@ bool AutoConfigTestPage::TestSoftwareEncoding()
 	if (wiz->type != AutoConfig::Type::Recording) {
 		obs_data_set_int(vencoder_settings, "keyint_sec", 2);
 		obs_data_set_int(vencoder_settings, "bitrate",
-				wiz->idealBitrate);
+				 wiz->idealBitrate);
 		obs_data_set_string(vencoder_settings, "rate_control", "CBR");
 		obs_data_set_string(vencoder_settings, "profile", "main");
 		obs_data_set_string(vencoder_settings, "preset", "veryfast");
@@ -540,18 +579,16 @@ bool AutoConfigTestPage::TestSoftwareEncoding()
 	/* -----------------------------------*/
 	/* connect signals                    */
 
-	auto on_stopped = [&] ()
-	{
+	auto on_stopped = [&]() {
 		unique_lock<mutex> lock(m);
 		cv.notify_one();
 	};
 
 	using on_stopped_t = decltype(on_stopped);
 
-	auto pre_on_stopped = [] (void *data, calldata_t *)
-	{
+	auto pre_on_stopped = [](void *data, calldata_t *) {
 		on_stopped_t &on_stopped =
-			*reinterpret_cast<on_stopped_t*>(data);
+			*reinterpret_cast<on_stopped_t *>(data);
 		on_stopped();
 	};
 
@@ -595,12 +632,13 @@ bool AutoConfigTestPage::TestSoftwareEncoding()
 	int i = 0;
 	int count = 1;
 
-	auto testRes = [&] (long double div, int fps_num, int fps_den,
-			bool force)
-	{
+	auto testRes = [&](int cy, int fps_num, int fps_den, bool force) {
 		int per = ++i * 100 / count;
 		QMetaObject::invokeMethod(this, "Progress", Q_ARG(int, per));
 
+		if (cy > baseCY)
+			return true;
+
 		/* no need for more than 3 tests max */
 		if (results.size() >= 3)
 			return true;
@@ -612,8 +650,8 @@ bool AutoConfigTestPage::TestSoftwareEncoding()
 
 		long double fps = ((long double)fps_num / (long double)fps_den);
 
-		int cx = int((long double)baseCX / div);
-		int cy = int((long double)baseCY / div);
+		int cx = int(((long double)baseCX / (long double)baseCY) *
+			     (long double)cy);
 
 		if (!force && wiz->type != AutoConfig::Type::Recording) {
 			int est = EstimateMinBitrate(cx, cy, fps_num, fps_den);
@@ -636,13 +674,13 @@ bool AutoConfigTestPage::TestSoftwareEncoding()
 		QString cxStr = QString::number(cx);
 		QString cyStr = QString::number(cy);
 
-		QString fpsStr = (fps_den > 1)
-			? QString::number(fps, 'f', 2)
-			: QString::number(fps, 'g', 2);
+		QString fpsStr = (fps_den > 1) ? QString::number(fps, 'f', 2)
+					       : QString::number(fps, 'g', 2);
 
-		QMetaObject::invokeMethod(this, "UpdateMessage",
-				Q_ARG(QString, QTStr(TEST_RES_VAL)
-					.arg(cxStr, cyStr, fpsStr)));
+		QMetaObject::invokeMethod(
+			this, "UpdateMessage",
+			Q_ARG(QString,
+			      QTStr(TEST_RES_VAL).arg(cxStr, cyStr, fpsStr)));
 
 		unique_lock<mutex> ul(m);
 		if (cancel)
@@ -650,7 +688,8 @@ bool AutoConfigTestPage::TestSoftwareEncoding()
 
 		if (!obs_output_start(output)) {
 			QMetaObject::invokeMethod(this, "Failure",
-					Q_ARG(QString, QTStr(TEST_RES_FAIL)));
+						  Q_ARG(QString,
+							QTStr(TEST_RES_FAIL)));
 			return false;
 		}
 
@@ -659,8 +698,8 @@ bool AutoConfigTestPage::TestSoftwareEncoding()
 		obs_output_stop(output);
 		cv.wait(ul);
 
-		int skipped = (int)video_output_get_skipped_frames(
-				obs_get_video());
+		int skipped =
+			(int)video_output_get_skipped_frames(obs_get_video());
 		if (force || skipped <= 10)
 			results.emplace_back(cx, cy, fps_num, fps_den);
 
@@ -668,24 +707,51 @@ bool AutoConfigTestPage::TestSoftwareEncoding()
 	};
 
 	if (wiz->specificFPSNum && wiz->specificFPSDen) {
-		count = 5;
-		if (!testRes(1.0, 0, 0, false)) return false;
-		if (!testRes(1.5, 0, 0, false)) return false;
-		if (!testRes(1.0 / 0.6, 0, 0, false)) return false;
-		if (!testRes(2.0, 0, 0, false)) return false;
-		if (!testRes(2.25, 0, 0, true)) return false;
+		count = 7;
+		if (!testRes(2160, 0, 0, false))
+			return false;
+		if (!testRes(1440, 0, 0, false))
+			return false;
+		if (!testRes(1080, 0, 0, false))
+			return false;
+		if (!testRes(720, 0, 0, false))
+			return false;
+		if (!testRes(480, 0, 0, false))
+			return false;
+		if (!testRes(360, 0, 0, false))
+			return false;
+		if (!testRes(240, 0, 0, true))
+			return false;
 	} else {
-		count = 10;
-		if (!testRes(1.0, 60, 1, false)) return false;
-		if (!testRes(1.0, 30, 1, false)) return false;
-		if (!testRes(1.5, 60, 1, false)) return false;
-		if (!testRes(1.5, 30, 1, false)) return false;
-		if (!testRes(1.0 / 0.6, 60, 1, false)) return false;
-		if (!testRes(1.0 / 0.6, 30, 1, false)) return false;
-		if (!testRes(2.0, 60, 1, false)) return false;
-		if (!testRes(2.0, 30, 1, false)) return false;
-		if (!testRes(2.25, 60, 1, false)) return false;
-		if (!testRes(2.25, 30, 1, true)) return false;
+		count = 14;
+		if (!testRes(2160, 60, 1, false))
+			return false;
+		if (!testRes(2160, 30, 1, false))
+			return false;
+		if (!testRes(1440, 60, 1, false))
+			return false;
+		if (!testRes(1440, 30, 1, false))
+			return false;
+		if (!testRes(1080, 60, 1, false))
+			return false;
+		if (!testRes(1080, 30, 1, false))
+			return false;
+		if (!testRes(720, 60, 1, false))
+			return false;
+		if (!testRes(720, 30, 1, false))
+			return false;
+		if (!testRes(480, 60, 1, false))
+			return false;
+		if (!testRes(480, 30, 1, false))
+			return false;
+		if (!testRes(360, 60, 1, false))
+			return false;
+		if (!testRes(360, 30, 1, false))
+			return false;
+		if (!testRes(240, 60, 1, false))
+			return false;
+		if (!testRes(240, 30, 1, true))
+			return false;
 	}
 
 	/* -----------------------------------*/
@@ -711,7 +777,7 @@ bool AutoConfigTestPage::TestSoftwareEncoding()
 	wiz->idealFPSDen = result.fps_den;
 
 	long double fUpperBitrate = EstimateUpperBitrate(
-			result.cx, result.cy, result.fps_num, result.fps_den);
+		result.cx, result.cy, result.fps_num, result.fps_den);
 
 	int upperBitrate = int(floor(fUpperBitrate / 50.0l) * 50.0l);
 
@@ -743,9 +809,10 @@ void AutoConfigTestPage::FindIdealHardwareResolution()
 		maxDataRate = 1280 * 720 * 30 + 1000;
 	}
 
-	auto testRes = [&] (long double div, int fps_num, int fps_den,
-			bool force)
-	{
+	auto testRes = [&](int cy, int fps_num, int fps_den, bool force) {
+		if (cy > baseCY)
+			return;
+
 		if (results.size() >= 3)
 			return;
 
@@ -756,15 +823,25 @@ void AutoConfigTestPage::FindIdealHardwareResolution()
 
 		long double fps = ((long double)fps_num / (long double)fps_den);
 
-		int cx = int((long double)baseCX / div);
-		int cy = int((long double)baseCY / div);
+		int cx = int(((long double)baseCX / (long double)baseCY) *
+			     (long double)cy);
 
 		long double rate = (long double)cx * (long double)cy * fps;
 		if (!force && rate > maxDataRate)
 			return;
 
-		int minBitrate = EstimateMinBitrate(cx, cy, fps_num, fps_den)
-			* 114 / 100;
+		AutoConfig::Encoder encType = wiz->streamingEncoder;
+		bool nvenc = encType == AutoConfig::Encoder::NVENC;
+
+		int minBitrate = EstimateMinBitrate(cx, cy, fps_num, fps_den);
+
+		/* most hardware encoders don't have a good quality to bitrate
+		 * ratio, so increase the minimum bitrate estimate for them.
+		 * NVENC currently is the exception because of the improvements
+		 * its made to its quality in recent generations. */
+		if (!nvenc)
+			minBitrate = minBitrate * 114 / 100;
+
 		if (wiz->type == AutoConfig::Type::Recording)
 			force = true;
 		if (force || wiz->idealBitrate >= minBitrate)
@@ -772,22 +849,28 @@ void AutoConfigTestPage::FindIdealHardwareResolution()
 	};
 
 	if (wiz->specificFPSNum && wiz->specificFPSDen) {
-		testRes(1.0, 0, 0, false);
-		testRes(1.5, 0, 0, false);
-		testRes(1.0 / 0.6, 0, 0, false);
-		testRes(2.0, 0, 0, false);
-		testRes(2.25, 0, 0, true);
+		testRes(2160, 0, 0, false);
+		testRes(1440, 0, 0, false);
+		testRes(1080, 0, 0, false);
+		testRes(720, 0, 0, false);
+		testRes(480, 0, 0, false);
+		testRes(360, 0, 0, false);
+		testRes(240, 0, 0, true);
 	} else {
-		testRes(1.0, 60, 1, false);
-		testRes(1.0, 30, 1, false);
-		testRes(1.5, 60, 1, false);
-		testRes(1.5, 30, 1, false);
-		testRes(1.0 / 0.6, 60, 1, false);
-		testRes(1.0 / 0.6, 30, 1, false);
-		testRes(2.0, 60, 1, false);
-		testRes(2.0, 30, 1, false);
-		testRes(2.25, 60, 1, false);
-		testRes(2.25, 30, 1, true);
+		testRes(2160, 60, 1, false);
+		testRes(2160, 30, 1, false);
+		testRes(1440, 60, 1, false);
+		testRes(1440, 30, 1, false);
+		testRes(1080, 60, 1, false);
+		testRes(1080, 30, 1, false);
+		testRes(720, 60, 1, false);
+		testRes(720, 30, 1, false);
+		testRes(480, 60, 1, false);
+		testRes(480, 30, 1, false);
+		testRes(360, 60, 1, false);
+		testRes(360, 30, 1, false);
+		testRes(240, 60, 1, false);
+		testRes(240, 30, 1, true);
 	}
 
 	int minArea = 960 * 540 + 1000;
@@ -821,9 +904,6 @@ void AutoConfigTestPage::TestStreamEncoderThread()
 		}
 	}
 
-	if (preferHardware && !softwareTested && wiz->hardwareEncodingAvailable)
-		FindIdealHardwareResolution();
-
 	if (!softwareTested) {
 		if (wiz->nvencAvailable)
 			wiz->streamingEncoder = AutoConfig::Encoder::NVENC;
@@ -835,6 +915,9 @@ void AutoConfigTestPage::TestStreamEncoderThread()
 		wiz->streamingEncoder = AutoConfig::Encoder::x264;
 	}
 
+	if (preferHardware && !softwareTested && wiz->hardwareEncodingAvailable)
+		FindIdealHardwareResolution();
+
 	QMetaObject::invokeMethod(this, "NextStage");
 }
 
@@ -877,13 +960,34 @@ void AutoConfigTestPage::TestRecordingEncoderThread()
 
 #define ENCODER_TEXT(x) "Basic.Settings.Output.Simple.Encoder." x
 #define ENCODER_SOFTWARE ENCODER_TEXT("Software")
-#define ENCODER_NVENC    ENCODER_TEXT("Hardware.NVENC")
-#define ENCODER_QSV      ENCODER_TEXT("Hardware.QSV")
-#define ENCODER_AMD      ENCODER_TEXT("Hardware.AMD")
+#define ENCODER_NVENC ENCODER_TEXT("Hardware.NVENC")
+#define ENCODER_QSV ENCODER_TEXT("Hardware.QSV")
+#define ENCODER_AMD ENCODER_TEXT("Hardware.AMD")
 
 #define QUALITY_SAME "Basic.Settings.Output.Simple.RecordingQuality.Stream"
 #define QUALITY_HIGH "Basic.Settings.Output.Simple.RecordingQuality.Small"
 
+void set_closest_res(int &cx, int &cy, struct obs_service_resolution *res_list,
+		     size_t count)
+{
+	int best_pixel_diff = 0x7FFFFFFF;
+	int start_cx = cx;
+	int start_cy = cy;
+
+	for (size_t i = 0; i < count; i++) {
+		struct obs_service_resolution &res = res_list[i];
+		int pixel_cx_diff = abs(start_cx - res.cx);
+		int pixel_cy_diff = abs(start_cy - res.cy);
+		int pixel_diff = pixel_cx_diff + pixel_cy_diff;
+
+		if (pixel_diff < best_pixel_diff) {
+			best_pixel_diff = pixel_diff;
+			cx = res.cx;
+			cy = res.cy;
+		}
+	}
+}
+
 void AutoConfigTestPage::FinalizeResults()
 {
 	ui->stackedWidget->setCurrentIndex(1);
@@ -891,8 +995,7 @@ void AutoConfigTestPage::FinalizeResults()
 
 	QFormLayout *form = results;
 
-	auto encName = [] (AutoConfig::Encoder enc) -> QString
-	{
+	auto encName = [](AutoConfig::Encoder enc) -> QString {
 		switch (enc) {
 		case AutoConfig::Encoder::x264:
 			return QTStr(ENCODER_SOFTWARE);
@@ -909,18 +1012,16 @@ void AutoConfigTestPage::FinalizeResults()
 		return QTStr(ENCODER_SOFTWARE);
 	};
 
-	auto newLabel = [this] (const char *str) -> QLabel *
-	{
+	auto newLabel = [this](const char *str) -> QLabel * {
 		return new QLabel(QTStr(str), this);
 	};
 
-	if (wiz->type != AutoConfig::Type::Recording) {
-		const char *serverType = wiz->customServer
-			? "rtmp_custom"
-			: "rtmp_common";
+	if (wiz->type == AutoConfig::Type::Streaming) {
+		const char *serverType = wiz->customServer ? "rtmp_custom"
+							   : "rtmp_common";
 
-		OBSService service = obs_service_create(serverType,
-				"temp_service", nullptr, nullptr);
+		OBSService service = obs_service_create(
+			serverType, "temp_service", nullptr, nullptr);
 		obs_service_release(service);
 
 		OBSData service_settings = obs_data_create();
@@ -929,44 +1030,66 @@ void AutoConfigTestPage::FinalizeResults()
 		obs_data_release(vencoder_settings);
 
 		obs_data_set_int(vencoder_settings, "bitrate",
-				wiz->idealBitrate);
+				 wiz->idealBitrate);
 
 		obs_data_set_string(service_settings, "service",
-				wiz->serviceName.c_str());
+				    wiz->serviceName.c_str());
 		obs_service_update(service, service_settings);
-		obs_service_apply_encoder_settings(service,
-				vencoder_settings, nullptr);
+		obs_service_apply_encoder_settings(service, vencoder_settings,
+						   nullptr);
 
-		wiz->idealBitrate = (int)obs_data_get_int(vencoder_settings,
-				"bitrate");
+		struct obs_service_resolution *res_list;
+		size_t res_count;
+		int maxFPS;
+		obs_service_get_supported_resolutions(service, &res_list,
+						      &res_count);
+		obs_service_get_max_fps(service, &maxFPS);
+
+		if (res_list) {
+			set_closest_res(wiz->idealResolutionCX,
+					wiz->idealResolutionCY, res_list,
+					res_count);
+		}
+		if (maxFPS) {
+			double idealFPS = (double)wiz->idealFPSNum /
+					  (double)wiz->idealFPSDen;
+			if (idealFPS > (double)maxFPS) {
+				wiz->idealFPSNum = maxFPS;
+				wiz->idealFPSDen = 1;
+			}
+		}
+
+		wiz->idealBitrate =
+			(int)obs_data_get_int(vencoder_settings, "bitrate");
 
 		if (!wiz->customServer)
 			form->addRow(
 				newLabel("Basic.AutoConfig.StreamPage.Service"),
 				new QLabel(wiz->serviceName.c_str(),
-					ui->finishPage));
+					   ui->finishPage));
 		form->addRow(newLabel("Basic.AutoConfig.StreamPage.Server"),
-			new QLabel(wiz->serverName.c_str(), ui->finishPage));
+			     new QLabel(wiz->serverName.c_str(),
+					ui->finishPage));
 		form->addRow(newLabel("Basic.Settings.Output.VideoBitrate"),
-			new QLabel(QString::number(wiz->idealBitrate),
-				ui->finishPage));
+			     new QLabel(QString::number(wiz->idealBitrate),
+					ui->finishPage));
 		form->addRow(newLabel(TEST_RESULT_SE),
-			new QLabel(encName(wiz->streamingEncoder),
-				ui->finishPage));
+			     new QLabel(encName(wiz->streamingEncoder),
+					ui->finishPage));
 	}
 
-	QString baseRes = QString("%1x%2").arg(
-			QString::number(wiz->baseResolutionCX),
-			QString::number(wiz->baseResolutionCY));
-	QString scaleRes = QString("%1x%2").arg(
-			QString::number(wiz->idealResolutionCX),
-			QString::number(wiz->idealResolutionCY));
+	QString baseRes =
+		QString("%1x%2").arg(QString::number(wiz->baseResolutionCX),
+				     QString::number(wiz->baseResolutionCY));
+	QString scaleRes =
+		QString("%1x%2").arg(QString::number(wiz->idealResolutionCX),
+				     QString::number(wiz->idealResolutionCY));
 
 	if (wiz->recordingEncoder != AutoConfig::Encoder::Stream ||
 	    wiz->recordingQuality != AutoConfig::Quality::Stream)
 		form->addRow(newLabel(TEST_RESULT_RE),
-			new QLabel(encName(wiz->recordingEncoder),
-				ui->finishPage));
+			     new QLabel(encName(wiz->recordingEncoder),
+					ui->finishPage));
 
 	QString recQuality;
 
@@ -980,21 +1103,20 @@ void AutoConfigTestPage::FinalizeResults()
 	}
 
 	form->addRow(newLabel("Basic.Settings.Output.Simple.RecordingQuality"),
-		new QLabel(recQuality, ui->finishPage));
+		     new QLabel(recQuality, ui->finishPage));
 
 	long double fps =
 		(long double)wiz->idealFPSNum / (long double)wiz->idealFPSDen;
 
-	QString fpsStr = (wiz->idealFPSDen > 1)
-		? QString::number(fps, 'f', 2)
-		: QString::number(fps, 'g', 2);
+	QString fpsStr = (wiz->idealFPSDen > 1) ? QString::number(fps, 'f', 2)
+						: QString::number(fps, 'g', 2);
 
 	form->addRow(newLabel("Basic.Settings.Video.BaseResolution"),
-		new QLabel(baseRes, ui->finishPage));
+		     new QLabel(baseRes, ui->finishPage));
 	form->addRow(newLabel("Basic.Settings.Video.ScaledResolution"),
-		new QLabel(scaleRes, ui->finishPage));
+		     new QLabel(scaleRes, ui->finishPage));
 	form->addRow(newLabel("Basic.Settings.Video.FPS"),
-		new QLabel(fpsStr, ui->finishPage));
+		     new QLabel(fpsStr, ui->finishPage));
 }
 
 #define STARTING_SEPARATOR \
@@ -1018,7 +1140,7 @@ void AutoConfigTestPage::NextStage()
 			started = true;
 		}
 
-		if (wiz->type == AutoConfig::Type::Recording) {
+		if (wiz->type != AutoConfig::Type::Streaming) {
 			stage = Stage::StreamEncoder;
 		} else if (!wiz->bandwidthTest) {
 			stage = Stage::BandwidthTest;
@@ -1061,8 +1183,7 @@ void AutoConfigTestPage::Progress(int percentage)
 }
 
 AutoConfigTestPage::AutoConfigTestPage(QWidget *parent)
-	: QWizardPage (parent),
-	  ui          (new Ui_AutoConfigTestPage)
+	: QWizardPage(parent), ui(new Ui_AutoConfigTestPage)
 {
 	ui->setupUi(this);
 	setTitle(QTStr("Basic.AutoConfig.TestPage"));
@@ -1089,8 +1210,17 @@ AutoConfigTestPage::~AutoConfigTestPage()
 
 void AutoConfigTestPage::initializePage()
 {
+	if (wiz->type == AutoConfig::Type::VirtualCam) {
+		wiz->idealResolutionCX = wiz->baseResolutionCX;
+		wiz->idealResolutionCY = wiz->baseResolutionCY;
+		wiz->idealFPSNum = 30;
+		wiz->idealFPSDen = 1;
+		stage = Stage::Finished;
+	} else {
+		stage = Stage::Starting;
+	}
+
 	setSubTitle(QTStr(SUBTITLE_TESTING));
-	stage = Stage::Starting;
 	softwareTested = false;
 	cancel = false;
 	DeleteLayout(results);
