@@ -1,4 +1,4 @@
-﻿/******************************************************************************
+/******************************************************************************
 Copyright (C) 2014 by Nibbles
 
 This program is free software: you can redistribute it and/or modify
@@ -28,19 +28,20 @@ FT_Library ft2_lib;
 
 OBS_DECLARE_MODULE()
 OBS_MODULE_USE_DEFAULT_LOCALE("text-freetype2", "en-US")
+MODULE_EXPORT const char *obs_module_description(void)
+{
+	return "FreeType2 text source";
+}
 
 uint32_t texbuf_w = 2048, texbuf_h = 2048;
 
-static struct obs_source_info freetype2_source_info = {
+static struct obs_source_info freetype2_source_info_v1 = {
 	.id = "text_ft2_source",
 	.type = OBS_SOURCE_TYPE_INPUT,
-	.output_flags = OBS_SOURCE_VIDEO |
-#ifdef _WIN32
-	                OBS_SOURCE_DEPRECATED |
-#endif
-	                OBS_SOURCE_CUSTOM_DRAW,
+	.output_flags = OBS_SOURCE_VIDEO | OBS_SOURCE_CAP_OBSOLETE |
+			OBS_SOURCE_CUSTOM_DRAW,
 	.get_name = ft2_source_get_name,
-	.create = ft2_source_create,
+	.create = ft2_source_create_v1,
 	.destroy = ft2_source_destroy,
 	.update = ft2_source_update,
 	.get_width = ft2_source_get_width,
@@ -48,6 +49,29 @@ static struct obs_source_info freetype2_source_info = {
 	.video_render = ft2_source_render,
 	.video_tick = ft2_video_tick,
 	.get_properties = ft2_source_properties,
+	.icon_type = OBS_ICON_TYPE_TEXT,
+};
+
+static struct obs_source_info freetype2_source_info_v2 = {
+	.id = "text_ft2_source",
+	.version = 2,
+	.type = OBS_SOURCE_TYPE_INPUT,
+	.output_flags = OBS_SOURCE_VIDEO |
+#ifdef _WIN32
+			OBS_SOURCE_DEPRECATED |
+#endif
+			OBS_SOURCE_CUSTOM_DRAW,
+	.get_name = ft2_source_get_name,
+	.create = ft2_source_create_v2,
+	.destroy = ft2_source_destroy,
+	.update = ft2_source_update,
+	.get_width = ft2_source_get_width,
+	.get_height = ft2_source_get_height,
+	.video_render = ft2_source_render,
+	.video_tick = ft2_video_tick,
+	.get_properties = ft2_source_properties,
+	.missing_files = ft2_missing_files,
+	.icon_type = OBS_ICON_TYPE_TEXT,
 };
 
 static bool plugin_initialized = false;
@@ -78,7 +102,8 @@ bool obs_module_load()
 		bfree(config_dir);
 	}
 
-	obs_register_source(&freetype2_source_info);
+	obs_register_source(&freetype2_source_info_v1);
+	obs_register_source(&freetype2_source_info_v2);
 
 	return true;
 }
@@ -101,14 +126,14 @@ static uint32_t ft2_source_get_width(void *data)
 {
 	struct ft2_source *srcdata = data;
 
-	return srcdata->cx;
+	return srcdata->cx + srcdata->outline_width;
 }
 
 static uint32_t ft2_source_get_height(void *data)
 {
 	struct ft2_source *srcdata = data;
 
-	return srcdata->cy;
+	return srcdata->cy + srcdata->outline_width;
 }
 
 static obs_properties_t *ft2_source_properties(void *unused)
@@ -124,39 +149,43 @@ static obs_properties_t *ft2_source_properties(void *unused)
 	//	Better/pixel shader outline/drop shadow
 	//	Some way to pull text files from network, I dunno
 
-	obs_properties_add_font(props, "font",
-		obs_module_text("Font"));
+	obs_properties_add_font(props, "font", obs_module_text("Font"));
 
-	obs_properties_add_text(props, "text",
-		obs_module_text("Text"), OBS_TEXT_MULTILINE);
+	obs_properties_add_text(props, "text", obs_module_text("Text"),
+				OBS_TEXT_MULTILINE);
 
 	obs_properties_add_bool(props, "from_file",
-		obs_module_text("ReadFromFile"));
+				obs_module_text("ReadFromFile"));
+
+	obs_properties_add_bool(props, "antialiasing",
+				obs_module_text("Antialiasing"));
 
 	obs_properties_add_bool(props, "log_mode",
-		obs_module_text("ChatLogMode"));
+				obs_module_text("ChatLogMode"));
 
-	obs_properties_add_path(props,
-		"text_file", obs_module_text("TextFile"),
-		OBS_PATH_FILE, obs_module_text("TextFileFilter"), NULL);
+	obs_properties_add_int(props, "log_lines",
+			       obs_module_text("ChatLogLines"), 1, 1000, 1);
 
-	obs_properties_add_color(props, "color1",
-		obs_module_text("Color1"));
+	obs_properties_add_path(props, "text_file", obs_module_text("TextFile"),
+				OBS_PATH_FILE,
+				obs_module_text("TextFileFilter"), NULL);
 
-	obs_properties_add_color(props, "color2",
-		obs_module_text("Color2"));
+	obs_properties_add_color_alpha(props, "color1",
+				       obs_module_text("Color1"));
 
-	obs_properties_add_bool(props, "outline",
-		obs_module_text("Outline"));
+	obs_properties_add_color_alpha(props, "color2",
+				       obs_module_text("Color2"));
+
+	obs_properties_add_bool(props, "outline", obs_module_text("Outline"));
 
 	obs_properties_add_bool(props, "drop_shadow",
-		obs_module_text("DropShadow"));
+				obs_module_text("DropShadow"));
 
 	obs_properties_add_int(props, "custom_width",
-		obs_module_text("CustomWidth"), 0, 4096, 1);
+			       obs_module_text("CustomWidth"), 0, 4096, 1);
 
 	obs_properties_add_bool(props, "word_wrap",
-		obs_module_text("WordWrap"));
+				obs_module_text("WordWrap"));
 
 	return props;
 }
@@ -169,7 +198,7 @@ static void ft2_source_destroy(void *data)
 		FT_Done_Face(srcdata->font_face);
 		srcdata->font_face = NULL;
 	}
-	
+
 	for (uint32_t i = 0; i < num_cache_slots; i++) {
 		if (srcdata->cacheglyphs[i] != NULL) {
 			bfree(srcdata->cacheglyphs[i]);
@@ -213,17 +242,22 @@ static void ft2_source_destroy(void *data)
 static void ft2_source_render(void *data, gs_effect_t *effect)
 {
 	struct ft2_source *srcdata = data;
-	if (srcdata == NULL) return;
+	if (srcdata == NULL)
+		return;
 
-	if (srcdata->tex == NULL || srcdata->vbuf == NULL) return;
-	if (srcdata->text == NULL || *srcdata->text == 0) return;
+	if (srcdata->tex == NULL || srcdata->vbuf == NULL)
+		return;
+	if (srcdata->text == NULL || *srcdata->text == 0)
+		return;
 
 	gs_reset_blend_state();
-	if (srcdata->outline_text) draw_outlines(srcdata);
-	if (srcdata->drop_shadow) draw_drop_shadow(srcdata);
+	if (srcdata->outline_text)
+		draw_outlines(srcdata);
+	if (srcdata->drop_shadow)
+		draw_drop_shadow(srcdata);
 
-	draw_uv_vbuffer(srcdata->vbuf, srcdata->tex,
-		srcdata->draw_effect, (uint32_t)wcslen(srcdata->text) * 6);
+	draw_uv_vbuffer(srcdata->vbuf, srcdata->tex, srcdata->draw_effect,
+			(uint32_t)wcslen(srcdata->text) * 6);
 
 	UNUSED_PARAMETER(effect);
 }
@@ -231,8 +265,10 @@ static void ft2_source_render(void *data, gs_effect_t *effect)
 static void ft2_video_tick(void *data, float seconds)
 {
 	struct ft2_source *srcdata = data;
-	if (srcdata == NULL) return;
-	if (!srcdata->from_file || !srcdata->text_file) return;
+	if (srcdata == NULL)
+		return;
+	if (!srcdata->from_file || !srcdata->text_file)
+		return;
 
 	if (os_gettime_ns() - srcdata->last_checked >= 1000000000) {
 		time_t t = get_modified_timestamp(srcdata->text_file);
@@ -243,7 +279,7 @@ static void ft2_video_tick(void *data, float seconds)
 				read_from_end(srcdata, srcdata->text_file);
 			else
 				load_text_from_file(srcdata,
-					srcdata->text_file);
+						    srcdata->text_file);
 			cache_glyphs(srcdata, srcdata->text);
 			set_up_vertex_buffer(srcdata);
 			srcdata->update_file = false;
@@ -262,7 +298,8 @@ static bool init_font(struct ft2_source *srcdata)
 {
 	FT_Long index;
 	const char *path = get_font_path(srcdata->font_name, srcdata->font_size,
-			srcdata->font_style, srcdata->font_flags, &index);
+					 srcdata->font_style,
+					 srcdata->font_flags, &index);
 	if (!path)
 		return false;
 
@@ -283,16 +320,24 @@ static void ft2_source_update(void *data, obs_data_t *settings)
 	uint32_t color[2];
 	uint32_t custom_width = 0;
 
-	const char *font_name  = obs_data_get_string(font_obj, "face");
+	const char *font_name = obs_data_get_string(font_obj, "face");
 	const char *font_style = obs_data_get_string(font_obj, "style");
-	uint16_t   font_size   = (uint16_t)obs_data_get_int(font_obj, "size");
-	uint32_t   font_flags  = (uint32_t)obs_data_get_int(font_obj, "flags");
+	uint16_t font_size = (uint16_t)obs_data_get_int(font_obj, "size");
+	uint32_t font_flags = (uint32_t)obs_data_get_int(font_obj, "flags");
 
 	if (!font_obj)
 		return;
 
+	srcdata->outline_width = 0;
+
 	srcdata->drop_shadow = obs_data_get_bool(settings, "drop_shadow");
 	srcdata->outline_text = obs_data_get_bool(settings, "outline");
+
+	if (srcdata->outline_text && srcdata->drop_shadow)
+		srcdata->outline_width = 6;
+	else if (srcdata->outline_text || srcdata->drop_shadow)
+		srcdata->outline_width = 4;
+
 	word_wrap = obs_data_get_bool(settings, "word_wrap");
 
 	color[0] = (uint32_t)obs_data_get_int(settings, "color1");
@@ -304,8 +349,7 @@ static void ft2_source_update(void *data, obs_data_t *settings)
 			srcdata->custom_width = custom_width;
 			vbuf_needs_update = true;
 		}
-	}
-	else {
+	} else {
 		if (srcdata->custom_width >= 100)
 			vbuf_needs_update = true;
 		srcdata->custom_width = 0;
@@ -324,17 +368,24 @@ static void ft2_source_update(void *data, obs_data_t *settings)
 
 	bool from_file = obs_data_get_bool(settings, "from_file");
 	bool chat_log_mode = obs_data_get_bool(settings, "log_mode");
+	uint32_t log_lines = (uint32_t)obs_data_get_int(settings, "log_lines");
 
+	if (srcdata->log_lines != log_lines) {
+		srcdata->log_lines = log_lines;
+		vbuf_needs_update = true;
+	}
 	srcdata->log_mode = chat_log_mode;
 
-	if (ft2_lib == NULL) goto error;
+	if (ft2_lib == NULL)
+		goto error;
+
+	const size_t texbuf_size = (size_t)texbuf_w * (size_t)texbuf_h;
 
 	if (srcdata->draw_effect == NULL) {
 		char *effect_file = NULL;
 		char *error_string = NULL;
 
-		effect_file =
-			obs_module_file("text_default.effect");
+		effect_file = obs_module_file("text_default.effect");
 
 		if (effect_file) {
 			obs_enter_graphics();
@@ -348,18 +399,27 @@ static void ft2_source_update(void *data, obs_data_t *settings)
 		}
 	}
 
-	if (srcdata->font_size != font_size ||
-	    srcdata->from_file != from_file)
+	if (srcdata->font_size != font_size || srcdata->from_file != from_file)
 		vbuf_needs_update = true;
 
+	const bool new_aa_setting = obs_data_get_bool(settings, "antialiasing");
+	const bool aa_changed = srcdata->antialiasing != new_aa_setting;
+	if (aa_changed) {
+		srcdata->antialiasing = new_aa_setting;
+		if (srcdata->texbuf != NULL) {
+			memset(srcdata->texbuf, 0, texbuf_size);
+		}
+		cache_standard_glyphs(srcdata);
+	}
+
 	srcdata->file_load_failed = false;
 	srcdata->from_file = from_file;
 
 	if (srcdata->font_name != NULL) {
-		if (strcmp(font_name,  srcdata->font_name)  == 0 &&
+		if (strcmp(font_name, srcdata->font_name) == 0 &&
 		    strcmp(font_style, srcdata->font_style) == 0 &&
 		    font_flags == srcdata->font_flags &&
-		    font_size  == srcdata->font_size)
+		    font_size == srcdata->font_size)
 			goto skip_font_load;
 
 		bfree(srcdata->font_name);
@@ -370,18 +430,17 @@ static void ft2_source_update(void *data, obs_data_t *settings)
 		vbuf_needs_update = true;
 	}
 
-	srcdata->font_name  = bstrdup(font_name);
+	srcdata->font_name = bstrdup(font_name);
 	srcdata->font_style = bstrdup(font_style);
-	srcdata->font_size  = font_size;
+	srcdata->font_size = font_size;
 	srcdata->font_flags = font_flags;
 
 	if (!init_font(srcdata) || srcdata->font_face == NULL) {
 		blog(LOG_WARNING, "FT2-text: Failed to load font %s",
-			srcdata->font_name);
+		     srcdata->font_name);
 		goto error;
-	}
-	else {
-		FT_Set_Pixel_Sizes(srcdata->font_face, 0, srcdata->font_size); 
+	} else {
+		FT_Set_Pixel_Sizes(srcdata->font_face, 0, srcdata->font_size);
 		FT_Select_Charmap(srcdata->font_face, FT_ENCODING_UNICODE);
 	}
 
@@ -389,7 +448,7 @@ static void ft2_source_update(void *data, obs_data_t *settings)
 		bfree(srcdata->texbuf);
 		srcdata->texbuf = NULL;
 	}
-	srcdata->texbuf = bzalloc(texbuf_w * texbuf_h);
+	srcdata->texbuf = bzalloc(texbuf_size);
 
 	if (srcdata->font_face)
 		cache_standard_glyphs(srcdata);
@@ -405,14 +464,15 @@ skip_font_load:
 			srcdata->text = NULL;
 
 			os_utf8_to_wcs_ptr(emptystr, strlen(emptystr),
-					&srcdata->text);
-			blog(LOG_WARNING, "FT2-text: Failed to open %s for "
-			                  "reading", tmp);
-		}
-		else {
+					   &srcdata->text);
+			blog(LOG_WARNING,
+			     "FT2-text: Failed to open %s for "
+			     "reading",
+			     tmp);
+		} else {
 			if (srcdata->text_file != NULL &&
-				strcmp(srcdata->text_file, tmp) == 0 &&
-				!vbuf_needs_update)
+			    strcmp(srcdata->text_file, tmp) == 0 &&
+			    !vbuf_needs_update)
 				goto error;
 
 			bfree(srcdata->text_file);
@@ -424,10 +484,10 @@ skip_font_load:
 				load_text_from_file(srcdata, tmp);
 			srcdata->last_checked = os_gettime_ns();
 		}
-	}
-	else {
+	} else {
 		const char *tmp = obs_data_get_string(settings, "text");
-		if (!tmp || !*tmp) goto error;
+		if (!tmp)
+			goto error;
 
 		if (srcdata->text != NULL) {
 			bfree(srcdata->text);
@@ -454,7 +514,8 @@ error:
 #define DEFAULT_FACE "Sans Serif"
 #endif
 
-static void *ft2_source_create(obs_data_t *settings, obs_source_t *source)
+static void *ft2_source_create(obs_data_t *settings, obs_source_t *source,
+			       int ver)
 {
 	struct ft2_source *srcdata = bzalloc(sizeof(struct ft2_source));
 	obs_data_t *font_obj = obs_data_create();
@@ -462,12 +523,23 @@ static void *ft2_source_create(obs_data_t *settings, obs_source_t *source)
 
 	init_plugin();
 
-	srcdata->font_size = 32;
+	const uint16_t font_size = ver == 1 ? 32 : 256;
+
+	srcdata->font_size = font_size;
 
 	obs_data_set_default_string(font_obj, "face", DEFAULT_FACE);
-	obs_data_set_default_int(font_obj, "size", 32);
+	obs_data_set_default_int(font_obj, "size", font_size);
+	obs_data_set_default_int(font_obj, "flags", 0);
+	obs_data_set_default_string(font_obj, "style", "");
 	obs_data_set_default_obj(settings, "font", font_obj);
 
+	obs_data_set_default_bool(settings, "antialiasing", true);
+	obs_data_set_default_bool(settings, "word_wrap", false);
+	obs_data_set_default_bool(settings, "outline", false);
+	obs_data_set_default_bool(settings, "drop_shadow", false);
+
+	obs_data_set_default_int(settings, "log_lines", 6);
+
 	obs_data_set_default_int(settings, "color1", 0xFFFFFFFF);
 	obs_data_set_default_int(settings, "color2", 0xFFFFFFFF);
 
@@ -477,3 +549,52 @@ static void *ft2_source_create(obs_data_t *settings, obs_source_t *source)
 
 	return srcdata;
 }
+
+static void *ft2_source_create_v1(obs_data_t *settings, obs_source_t *source)
+{
+	return ft2_source_create(settings, source, 1);
+}
+
+static void *ft2_source_create_v2(obs_data_t *settings, obs_source_t *source)
+{
+	return ft2_source_create(settings, source, 2);
+}
+
+static void missing_file_callback(void *src, const char *new_path, void *data)
+{
+	struct ft2_source *s = src;
+
+	obs_source_t *source = s->src;
+	obs_data_t *settings = obs_source_get_settings(source);
+	obs_data_set_string(settings, "text_file", new_path);
+	obs_source_update(source, settings);
+	obs_data_release(settings);
+
+	UNUSED_PARAMETER(data);
+}
+
+static obs_missing_files_t *ft2_missing_files(void *data)
+{
+	struct ft2_source *s = data;
+	obs_missing_files_t *files = obs_missing_files_create();
+
+	obs_source_t *source = s->src;
+	obs_data_t *settings = obs_source_get_settings(source);
+
+	bool read = obs_data_get_bool(settings, "from_file");
+	const char *path = obs_data_get_string(settings, "text_file");
+
+	if (read && strcmp(path, "") != 0) {
+		if (!os_file_exists(path)) {
+			obs_missing_file_t *file = obs_missing_file_create(
+				path, missing_file_callback,
+				OBS_MISSING_FILE_SOURCE, s->src, NULL);
+
+			obs_missing_files_add_file(files, file);
+		}
+	}
+
+	obs_data_release(settings);
+
+	return files;
+}
