@@ -46,6 +46,7 @@ using namespace DShow;
 #define COLOR_SPACE       "color_space"
 #define COLOR_RANGE       "color_range"
 #define DEACTIVATE_WNS    "deactivate_when_not_showing"
+#define AUTOROTATION      "autorotation"
 
 #define TEXT_INPUT_NAME     obs_module_text("VideoCaptureDevice")
 #define TEXT_DEVICE         obs_module_text("Device")
@@ -64,6 +65,7 @@ using namespace DShow;
 #define TEXT_BUFFERING_ON   obs_module_text("Buffering.Enable")
 #define TEXT_BUFFERING_OFF  obs_module_text("Buffering.Disable")
 #define TEXT_FLIP_IMAGE     obs_module_text("FlipVertically")
+#define TEXT_AUTOROTATION   obs_module_text("Autorotation")
 #define TEXT_AUDIO_MODE     obs_module_text("AudioOutputMode")
 #define TEXT_MODE_CAPTURE   obs_module_text("AudioOutputMode.Capture")
 #define TEXT_MODE_DSOUND    obs_module_text("AudioOutputMode.DirectSound")
@@ -181,6 +183,7 @@ struct DShowInput {
 	bool deviceHasSeparateAudioFilter = false;
 	bool flip = false;
 	bool active = false;
+	bool autorotation = true;
 
 	Decoder audio_decoder;
 	Decoder video_decoder;
@@ -522,7 +525,7 @@ void DShowInput::OnVideoData(const VideoConfig &config, unsigned char *data,
 			     size_t size, long long startTime,
 			     long long endTime, long rotation)
 {
-	if (rotation != lastRotation) {
+	if (autorotation && rotation != lastRotation) {
 		lastRotation = rotation;
 		obs_source_set_async_rotation(source, rotation);
 	}
@@ -706,7 +709,7 @@ static inline bool FormatMatches(VideoFormat left, VideoFormat right)
 	       left == right;
 }
 
-static inline bool ResolutionValid(string res, int &cx, int &cy)
+static inline bool ResolutionValid(const string &res, int &cx, int &cy)
 {
 	if (!res.size())
 		return false;
@@ -813,7 +816,7 @@ static bool ResolutionAvailable(const VideoDevice &dev, int cx, int cy)
 }
 
 static bool DetermineResolution(int &cx, int &cy, obs_data_t *settings,
-				VideoDevice dev)
+				VideoDevice &dev)
 {
 	const char *res = obs_data_get_autoselect_string(settings, RESOLUTION);
 	if (obs_data_has_autoselect_value(settings, RESOLUTION) &&
@@ -836,7 +839,8 @@ static long long GetOBSFPS();
 static inline bool IsDelayedDevice(const VideoConfig &config)
 {
 	return config.format > VideoFormat::MJPEG ||
-	       wstrstri(config.name.c_str(), L"elgato") != NULL ||
+	       (wstrstri(config.name.c_str(), L"elgato") != NULL &&
+		wstrstri(config.name.c_str(), L"facecam") == NULL) ||
 	       wstrstri(config.name.c_str(), L"stream engine") != NULL;
 }
 
@@ -868,6 +872,7 @@ bool DShowInput::UpdateVideoConfig(obs_data_t *settings)
 	string video_device_id = obs_data_get_string(settings, VIDEO_DEVICE_ID);
 	deactivateWhenNotShowing = obs_data_get_bool(settings, DEACTIVATE_WNS);
 	flip = obs_data_get_bool(settings, FLIP_IMAGE);
+	autorotation = obs_data_get_bool(settings, AUTOROTATION);
 
 	DeviceId id;
 	if (!DecodeDeviceId(id, video_device_id.c_str())) {
@@ -928,8 +933,8 @@ bool DShowInput::UpdateVideoConfig(obs_data_t *settings)
 		interval = best_interval;
 	}
 
-	videoConfig.name = id.name.c_str();
-	videoConfig.path = id.path.c_str();
+	videoConfig.name = id.name;
+	videoConfig.path = id.path;
 	videoConfig.useDefaultConfig = resType == ResType_Preferred;
 	videoConfig.cx = cx;
 	videoConfig.cy_abs = abs(cy);
@@ -996,8 +1001,8 @@ bool DShowInput::UpdateAudioConfig(obs_data_t *settings)
 		if (!DecodeDeviceId(id, audio_device_id.c_str()))
 			return false;
 
-		audioConfig.name = id.name.c_str();
-		audioConfig.path = id.path.c_str();
+		audioConfig.name = id.name;
+		audioConfig.path = id.path;
 
 	} else if (!deviceHasAudio) {
 		return true;
@@ -1191,6 +1196,7 @@ static void GetDShowDefaults(obs_data_t *settings)
 	obs_data_set_default_string(settings, COLOR_RANGE, "default");
 	obs_data_set_default_int(settings, AUDIO_OUTPUT_MODE,
 				 (int)AudioMode::Capture);
+	obs_data_set_default_bool(settings, AUTOROTATION, true);
 }
 
 struct Resolution {
@@ -1269,7 +1275,7 @@ static const FPSFormat validFPSFormats[] = {
 static bool DeviceIntervalChanged(obs_properties_t *props, obs_property_t *p,
 				  obs_data_t *settings);
 
-static bool TryResolution(VideoDevice &dev, string res)
+static bool TryResolution(const VideoDevice &dev, const string &res)
 {
 	int cx, cy;
 	if (!ConvertRes(cx, cy, res.c_str()))
@@ -1279,7 +1285,7 @@ static bool TryResolution(VideoDevice &dev, string res)
 }
 
 static bool SetResolution(obs_properties_t *props, obs_data_t *settings,
-			  string res, bool autoselect = false)
+			  const string &res, bool autoselect = false)
 {
 	if (autoselect)
 		obs_data_set_autoselect_string(settings, RESOLUTION,
@@ -1940,6 +1946,8 @@ static obs_properties_t *GetDShowProperties(void *obj)
 
 	obs_properties_add_bool(ppts, FLIP_IMAGE, TEXT_FLIP_IMAGE);
 
+	obs_properties_add_bool(ppts, AUTOROTATION, TEXT_AUTOROTATION);
+
 	/* ------------------------------------- */
 	/* audio settings */
 
